/*
 * Copyright © 2021-2022 Koherence, LLC (www.koherence.com). 
 * All Rights Reserved.
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it 
 * and/or modify it under the terms of the GNU Affero General Public License 
 * as published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with the Entangle ATSC 3.0 Base Stack. If not, see 
 * <https://www.gnu.org/licenses/>.
 */

#ifndef ENATSC3_SPECDEFS_H
#define ENATSC3_SPECDEFS_H

// This file contains structures and values corresponding to the A/3*
// specifications.

// ALP packet type, from A/330 5.2
typedef enum
{
    ENATSC3ALPPacketType_IPV4 = 0,
    ENATSC3ALPPacketType_CompressedIP = 2,
    ENATSC3ALPPacketType_LinkLayerSignaling = 4,
    ENATSC3ALPPacketType_PacketTypeExtension = 6,
    ENATSC3ALPPacketType_MPEG2TS = 7,
} ENATSC3ALPPacketType;

typedef enum
{
    ENATSC3ALPSignalingType_LinkMappingTable = 1,
    ENATSC3ALPSignalingType_ROHCUDescriptionTable = 2
} ENATSC3ALPSignalingType;

typedef enum
{
    ENATSC3ALPSignalingFormat_Binary = 0,
    ENATSC3ALPSignalingFormat_XML = 1,
    ENATSC3ALPSignalingFormat_JSON = 2
} ENATSC3ALPSignalingFormat;

typedef enum
{
    ENATSC3ALPSignalingEncoding_Uncompressed = 0,
    ENATSC3ALPSignalingEncoding_DEFLATE = 1
} ENATSC3ALPSignalingEncoding;

typedef struct
{
    uint32_t signaling_type;
    uint32_t signaling_type_extension;
    uint32_t signaling_version;
    uint32_t signaling_format;
    uint32_t signaling_encoding;
} __attribute__((__packed__)) ENATSC3ALPLinkLayerSignalingHeader;

// The following is the four-byte header for LLS tables. See A/331 table 6.1
typedef struct
{
    uint8_t lls_table_id;
    uint8_t lls_group_id;
    uint8_t group_count_minus1;
    uint8_t lls_table_version;
} __attribute__((__packed__)) ENATSC3LLSTableHeader;

#endif
