/*
 * Copyright © 2020-2022 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ATSC3ESG_SCHEDULEFRAGMENT_H
#define ATSC3ESG_SCHEDULEFRAGMENT_H

typedef struct
{
    EnXMLParserNodeBase base;

    int64_t startTime;
    int64_t endTime;
    int32_t duration;
} ATSC3ESGScheduleFragmentPresentationWindowNode;

#define ATSC3ESGScheduleFragmentPresentationWindowNodeAttribute_startTime 0x0001
#define ATSC3ESGScheduleFragmentPresentationWindowNodeAttribute_endTime   0x0002
#define ATSC3ESGScheduleFragmentPresentationWindowNodeAttribute_duration  0x0004

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t idRefXMLOffset;
    int32_t idRefXMLLen;

    ATSC3ESGScheduleFragmentPresentationWindowNode presentationWindow;
} ATSC3ESGScheduleFragmentContentReferenceNode;

#define ATSC3ESGScheduleFragmentContentReferenceNodeAttribute_idRef     0x0001
#define ATSC3ESGScheduleFragmentContentReferenceChildNode_PresentationWindow 0x0002

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t idRefXMLOffset;
    int32_t idRefXMLLen;
} ATSC3ESGScheduleFragmentServiceReferenceNode;

#define ATSC3ESGScheduleFragmentServiceReferenceNodeAttribute_idRef     0x0001

typedef struct
{
    EnXMLParserNodeBase base;

    char *pXML;
    int32_t sXML;

    uint32_t version;
    int32_t idXMLOffset;
    int32_t idXMLLen;

    ATSC3ESGScheduleFragmentServiceReferenceNode serviceReference;

    EnXMLParserDataOffsetArray contentReferences;

    EnXMLParserDataBuffer dataBuffer;
} ATSC3ESGScheduleFragmentNode;

#define ATSC3ESGScheduleFragmentNodeAttribute_id                 0x0001
#define ATSC3ESGScheduleFragmentNodeAttribute_version            0x0002
#define ATSC3ESGScheduleFragmentChildNode_ServiceReference       0x0004
#define ATSC3ESGScheduleFragmentChildNode_ContentReference       0x0008

typedef enum
{
    ATSC3ESGScheduleFragmentNodeType_Invalid = 0,
    ATSC3ESGScheduleFragmentNodeType_ScheduleFragment = 1,
    ATSC3ESGScheduleFragmentNodeType_ServiceReference = 2,
    ATSC3ESGScheduleFragmentNodeType_ContentReference = 3,
    ATSC3ESGScheduleFragmentNodeType_PresentationWindow = 4,
} ATSC3ESGScheduleFragmentNodeType;

int32_t
ATSC3ESGScheduleFragmentParse(const char *pXML, int32_t sXML,
    ATSC3ESGScheduleFragmentNode *pParsedScheduleFragment);

int32_t
ATSC3ESGScheduleFragmentFree(
    ATSC3ESGScheduleFragmentNode *pParsedScheduleFragment);

#endif
