/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ATSC3LLS_ONSCREENMESSAGE_H
#define ATSC3LLS_ONSCREENMESSAGE_H

// See A/331 6.6

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t bsid;                      // Identifier of the broadcast stream
    uint32_t serviceId;                 // Identifier of the service within the 
                                        // scope of the broadcast tream that 
                                        // the notificiation applies.
    uint32_t serviceIdRange;            // Identifier of a range of serviceId 
                                        // that this KeepScreenClear applies.
    int64_t notificationDurationMilliseconds; // Indicates duration of this
                                        // KeepScreenClear applies
    int32_t kscFlag;                    // Indicates the status of 
                                        // KeepScreenClear
    uint32_t version;                   // The version of KeepScreenClear

} ATSCLLSOSMN_KeepScreenClearNode;

#define ATSC3LLSOSMNKeepScreenClearNodeAttribute_bsid           0x0001
#define ATSC3LLSOSMNKeepScreenClearNodeAttribute_serviceId      0x0002
#define ATSC3LLSOSMNKeepScreenClearNodeAttribute_serviceIdRange 0x0004
#define ATSC3LLSOSMNKeepScreenClearNodeAttribute_notificationDuration 0x0008
#define ATSC3LLSOSMNKeepScreenClearNodeAttribute_kscFlag        0x0010
#define ATSC3LLSOSMNKeepScreenClearNodeAttribute_version        0x0020

typedef struct
{
    EnXMLParserNodeBase base;

    EnXMLParserDataOffsetArray keepScreenClears; // Array of 
                                                // ATSCLLSOSMN_KeepScreenClearNode
    EnXMLParserDataBuffer dataBuffer;
} ATSC3LLSOSMN_OnscreenMessageNotificationNode;

#define ATSC3LLSOSMNOnscreenMessageNotificationNodeAttribute_XMLNS       0x0001
#define ATSC3LLSOSMNOnscreenMessageNotificationChildNode_KeepScreenClear 0x0002

typedef enum
{
    ATSC3LLSOSMNNodeType_OnscreenMessageNotification = 1,
    ATSC3LLSOSMNNodeType_KeepScreenClear = 2,
} ATSC3LLSOSMNNodeType;

ENATSC3Status
ATSC3LLSParseOnscreenMessageNotification(const char *pXML, int32_t sXML,
    ATSC3LLSOSMN_OnscreenMessageNotificationNode 
        *pParsedOnscreenMessageNotification);

ENATSC3Status
ATSC3LLSFreeParsedOnscreenMessageNotification(
    ATSC3LLSOSMN_OnscreenMessageNotificationNode 
        *pParsedOnscreenMessageNotification);

#endif
