/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ATSC3LLS_SYSTEMTIME_H
#define ATSC3LLS_SYSTEMTIME_H

// See A/331 6.4
typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t currentUtcOffset;  // The current offset in whole seconds between
                                // TAI and UTC.
    uint32_t ptpPrepend;        // Signals the upper 16 bits of the 48-bit 
                                // count of PTP seconds
    uint32_t leap59;            // indicates a pending 59-second leap second 
                                // event
    uint32_t leap61;            // Indicates a pending 61-second leap second
                                // event
    int32_t utcLocalOffsetDataOffset; // Indicates the offset between the local time
                                // zone of the originating broadcast station
                                // and UTC.
    uint32_t dsStatus;          // Indicates that Daylight Savings Time is in
                                // effect.
    uint32_t dsDayOfMonth;      // Indicates the local day of the month on 
                                // which transition into or out of daylight
                                // savings time is to occur.
    uint32_t dsHour;            // Indicates the local hour at which the
                                // transition into or out of daylight savings
                                // time is to occur (0-24)
    
    EnXMLParserDataBuffer dataBuffer;
} ATSC3LLSSystemTime_SystemTimeNode;

typedef enum
{
    ATSC3LLSSystemTimeNodeType_Invalid = 0,
    ATSC3LLSSystemTimeNodeType_SystemTime = 1,
} ATSC3LLSSystemTimeNodeType;

#define ATSC3LLSSystemTimeNodeAttribute_XMLNS                           0x0001
#define ATSC3LLSSystemTimeNodeAttribute_CurrentUtcOffset                0x0002
#define ATSC3LLSSystemTimeNodeAttribute_PtpPrepend                      0x0004
#define ATSC3LLSSystemTimeNodeAttribute_Leap59                          0x0008
#define ATSC3LLSSystemTimeNodeAttribute_Leap61                          0x0010
#define ATSC3LLSSystemTimeNodeAttribute_UtcLocalOffset                  0x0020
#define ATSC3LLSSystemTimeNodeAttribute_DsStatus                        0x0040
#define ATSC3LLSSystemTimeNodeAttribute_DsDayOfMonth                    0x0080
#define ATSC3LLSSystemTimeNodeAttribute_DsHour                          0x0100

ENATSC3Status
ATSC3LLSParseSystemTime(const char *pXML, int32_t sXML, 
    ATSC3LLSSystemTime_SystemTimeNode *pParsedSystemTime);

ENATSC3Status
ATSC3LLSFreeParsedSystemTime(
    ATSC3LLSSystemTime_SystemTimeNode *pParsedSystemTime);

#endif
