/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com). 
 * All Rights Reserved.
 */

#ifndef ENATSC3MMTPSLS_USBD_H
#define ENATSC3MMTPSLS_USBD_H

enum 
{
    ATSC3MMTPLLSUSBDComponentType_Audio = 0,
    ATSC3MMTPLLSUSBDComponentType_Video = 1,
    ATSC3MMTPLLSUSBDComponentType_Caption = 2,
};

enum
{
    ATSC3MMTPLLSUSBDComponentRole_AudioCompleteMain = 0,
    ATSC3MMTPLLSUSBDComponentRole_AudioMusicAndEffects = 1,
    ATSC3MMTPLLSUSBDComponentRole_AudioDialog = 2,
    ATSC3MMTPLLSUSBDComponentRole_AudioCommentary = 3,
    ATSC3MMTPLLSUSBDComponentRole_AudioVisuallyImpaired = 4,
    ATSC3MMTPLLSUSBDComponentRole_AudioHearingImparied = 5,
    ATSC3MMTPLLSUSBDComponentRole_AudioVoiceOver=6,

    ATSC3MMTPLLSUSBDComponentRole_VideoPrimary = 0,

    ATSC3MMTPLLSUSBDComponentRole_CaptionNormal = 0,
    ATSC3MMTPLLSUSBDComponentRole_CaptionEasyReader = 1
};

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t componentType;
    uint32_t componentRole;
    int32_t componentProtectedFlag;
    int32_t componentIdDataOffset;
    int32_t componentNameDataOffset;
} ATSC3MMTPSLSUSBDComponentInfoNode;

#define ATSC3MMTPSLSUSBDComponentInfoNodeAttribute_componentType         0x0001
#define ATSC3MMTPSLSUSBDComponentInfoNodeAttribute_componentRole         0x0002
#define ATSC3MMTPSLSUSBDComponentInfoNodeAttribute_componentProtectedFlag 0x0004
#define ATSC3MMTPSLSUSBDComponentInfoNodeAttribute_componentId           0x0008
#define ATSC3MMTPSLSUSBDComponentInfoNodeAttribute_componentName         0x0010

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t mmtPackageIdDataOffset;
} ATSC3MMTPSLSUSBDMPUComponentNode;

#define ATSC3MMTPSLSUSBDMPUComponentNodeAttribute_mmtPackageId   0x0001

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t serviceGenre;
    int32_t serviceIconDataOffset;
} ATSC3MMTPSLSUSBDChannelNode;

#define ATSC3MMTPSLSUSBDChannelNodeAttribute_serviceGenre        0x0001
#define ATSC3MMTPSLSUSBDChannelNodeAttribute_serviceIcon         0x0002

typedef struct
{
    EnXMLParserNodeBase base;

    uint32_t serviceId;
    int32_t serviceStatus;

    ATSC3MMTPSLSUSBDChannelNode channel;
    ATSC3MMTPSLSUSBDMPUComponentNode mpuComponent;
    ATSC3MMTPSLSUSBDComponentInfoNode vComponentInfos[8];
    int32_t nComponentInfos;
} ATSC3MMTPSLSUSBDUserServiceDescriptionNode;

#define ATSC3MMTPSLSUSBDUserServiceDescriptionNodeAttribute_serviceId    0x0001
#define ATSC3MMTPSLSUSBDUserServiceDescriptionNodeAttribute_serviceStatus 0x0002
#define ATSC3MMTPSLSUSBDUserServiceDescriptionChildNode_Channel          0x0004
#define ATSC3MMTPSLSUSBDUserServiceDescriptionChildNode_MPUComponent     0x0008
#define ATSC3MMTPSLSUSBDUserServiceDescriptionChildNode_ComponentInfo    0x0010

typedef struct
{
    EnXMLParserNodeBase base;

    ATSC3MMTPSLSUSBDUserServiceDescriptionNode userServiceDescription;

    EnXMLParserDataBuffer dataBuffer;
} ATSC3MMTPSLSUSBDBundleDescriptionNode;

#define ATSC3MMTPSLSUSBDBundleDescriptionRouteNodeAttribute_XMLNS   0x0001
#define ATSC3MMTPSLSUSBDBundleDescriptionRouteChildNode_userServiceDescription 0x0002

typedef enum
{
    ATSC3MMTPSLSUSBDNodeType_Invalid = 0,
    ATSC3MMTPSLSUSBDNodeType_BundleDescriptionMMT = 1,
    ATSC3MMTPSLSUSBDNodeType_UserServiceDescription = 2,
    ATSC3MMTPSLSUSBDNodeType_Channel = 3,
    ATSC3MMTPSLSUSBDNodeType_MPUComponent = 4,
    ATSC3MMTPSLSUSBDNodeType_ComponentInfo = 5,
} ATSC3MMTPSLSUSBDNodeType;

ENATSC3Status
ENATSC3MMTPSLSUSBDParse(const char *pXML, int32_t sXML,
    ATSC3MMTPSLSUSBDBundleDescriptionNode *pParsedUSBD);

ENATSC3Status
ENATSC3MMTPSLSUSBDFree(ATSC3MMTPSLSUSBDBundleDescriptionNode *pParsedUSBD);

#endif
