/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com). 
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it
 * and/or modify it under the terms of the GNU Affero General Public License
 * version 3 as published by the Free Software Foundation with the addition of 
 * the following permission added to Section 15 as permitted in Section 7(a): 
 * FOR ANY PART OF THE COVERED WORK IN WHICH THE COPYRIGHT IS OWNED BY 
 * KOHERENCE, LLC, KOHERENCE, LLC DISCLAIMS THE WARRANTY OF NON INFRINGEMENT 
 * OF THIRD PARTY RIGHTS.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write 
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, 
 * Boston, MA 02110-1301 USA.
 *
 * You can contact Koherence, LLC at email address info@koherence.com
 */

#ifndef ENATSC3_SLS_USBD_H
#define ENATSC3_SLS_USBD_H

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t basePatternDataOffset;
} ENATSC3SLSUSBDBasePatternNode;

#define ENATSC3SLSUSBDBasePatternNodeAttribute_basePattern        0x0001

typedef struct
{
    EnXMLParserNodeBase base;

    EnXMLParserDataOffsetArray basePatterns;    // Array of
                                                // EnXMLParserDataOffsetArray
} ENATSC3SLSUSBDBroadcastAppServiceNode;

#define ENATSC3SLSUSBDBroadcastAppServiceNodeAttribute_basePattern        0x0001

typedef struct
{
    EnXMLParserNodeBase base;

    EnXMLParserDataOffsetArray basePatterns;    // Array of
                                                // EnXMLParserDataOffsetArray
} ENATSC3SLSUSBDUnicastAppServiceNode;

#define ENATSC3SLSUSBDUnicastAppServiceNodeAttribute_basePattern  0x0001

typedef struct
{
    EnXMLParserNodeBase base;

    EnXMLParserDataOffsetArray broadcastAppServices; // Array of
                                        // ENATSC3SLSUSBDBroadcastAppServiceNode
    EnXMLParserDataOffsetArray unicastAppServices;  // Array of
                                        // ENATSC3SLSUSBDUnicastAppServiceNode
} ENATSC3SLSUSBDDeliveryMethodNode;

#define ENATSC3SLSUSBDDeliveryMethodChildNode_broadcastAppService 0x0001
#define ENATSC3SLSUSBDDeliveryMethodChildNode_unicastAppService   0x0002

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t nameDataOffset;
    int32_t langDataOffset;
} ENATSC3SLSUSBDNameNode;

#define ENATSC3SLSUSBDNameNodeAttribute_name                      0x0001
#define ENATSC3SLSUSBDNameNodeAttribute_lang                      0x0002

typedef struct
{
    EnXMLParserNodeBase base;

    int32_t globalServiceIDDataOffset;
    EnXMLParserDataOffsetArray names;   // Array of
                                        // ENATSC3SLSUSBDNameNode
    EnXMLParserDataOffsetArray deliveryMethods; // Array of
                                        // ENATSC3SLSUSBDDeliveryMethodNode
    int32_t STSIDUriDataOffset;
    uint32_t serviceId;
} ENATSC3SLSUSBDUserServiceDescriptionNode;

#define ENATSC3SLSUSBDUserServiceDescriptionNodeAttribute_globalServiceID 0x0001
#define ENATSC3SLSUSBDUserServiceDescriptionNodeAttribute_serviceId       0x0002
#define ENATSC3SLSUSBDUserServiceDescriptionNodeAttribute_sTSIDUri        0x0004
#define ENATSC3SLSUSBDUserServiceDescriptionChildNode_Name                0x0008
#define ENATSC3SLSUSBDUserServiceDescriptionChildNode_DeliveryMethod      0x0010

typedef struct
{
    EnXMLParserNodeBase base;

    ENATSC3SLSUSBDUserServiceDescriptionNode userServiceDescription;

    EnXMLParserDataBuffer dataBuffer;
} ENATSC3SLSUSBDBundleDescriptionNode;

#define ENATSC3SLSUSBDBundleDescriptionRouteNodeAttribute_XMLNS   0x0001
#define ENATSC3SLSUSBDBundleDescriptionRouteChildNode_userServiceDescription 0x0002

typedef enum
{
    ENATSC3SLSUSBDNodeType_Invalid = 0,
    ENATSC3SLSUSBDNodeType_BundleDescriptionROUTE = 1,
    ENATSC3SLSUSBDNodeType_UserServiceDescription = 2,
    ENATSC3SLSUSBDNodeType_Name = 3,
    ENATSC3SLSUSBDNodeType_DeliveryMethod = 4,
    ENATSC3SLSUSBDNodeType_BroadcastAppService = 5, 
    ENATSC3SLSUSBDNodeType_UnicastAppService = 5, 
    ENATSC3SLSUSBDNodeType_BasePattern = 6
} ENATSC3SLSUSBDNodeType;

ENATSC3Status
ENATSC3SLSUSBDParse(const char *pXML, int32_t sXML,
    ENATSC3SLSUSBDBundleDescriptionNode *pParsedUSBD);

ENATSC3Status
ENATSC3SLSUSBDFree(ENATSC3SLSUSBDBundleDescriptionNode *pParsedUSBD);

#endif
