/*
 * Copyright © 2018-2022 Koherence, LLC (www.koherence.com). 
 * All Rights Reserved.
 *
 * This file is part of the Entangle ATSC 3.0 Base Stack.
 *
 * The Entangle ATSC 3.0 Base Stack is free software: you can redistribute it 
 * and/or modify it under the terms of the GNU Affero General Public License 
 * as published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 *
 * The Entangle ATSC 3.0 Base Stack is distributed in the hope that it will be 
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Affero 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License 
 * along with the Entangle ATSC 3.0 Base Stack. If not, see 
 * <https://www.gnu.org/licenses/>. 
 */

#ifndef LIBENDEBUG_STUB_H
#define LIBENDEBUG_STUB_H

typedef enum
{
    EnLogFile_Debug = 1,
    EnLogFile_Info = 2,
    EnLogFile_Error = 3,
    EnLogFile_Player = 100,
    EnLogFile_TSSource = 101,
} EnLogFile;

typedef enum
{
    EnLogLevel_None = 0,
    EnLogLevel_Debug = 1,
    EnLogLevel_Info = 2,
    EnLogLevel_Error = 3,
} EnLogLevel;

typedef struct
{
    int32_t nLogRequestsCreated;
} EnLogStatistics;

int32_t
EnLogInitialize(int32_t fLogFiles);

int32_t 
EnLogShutdown(void);

int32_t
EnLogFlush(void);

void
EnLogSetNoConsole(int32_t fNoConsole);

int32_t
EnLogGetStatistics(EnLogStatistics *pStatistics);

int32_t 
EnLog(int32_t fSendToConsole, int32_t fLogToFile, EnLogFile log, 
    const char *sys, const char *subsys, int32_t line, 
    const char *format, ...);

int32_t
EnLogNoPrefix(int32_t fSendToConsole, int32_t fLogToFile, EnLogFile log, 
    const char *format, ...);

int32_t 
EnLogThrottle(struct timespec *pTNextMessage, struct timespec throttle,
    int32_t fSendToConsole, int32_t fLogToFile, EnLogFile log, 
    const char *sys, const char *subsys, int32_t line, 
    const char *format, ...);

int32_t
EnLogNoPrefixThrottle(struct timespec *pTNextMessage, 
    struct timespec throttle, int32_t fSendToConsole, int32_t fLogToFile, 
    EnLogFile log, const char *format, ...);

#define EnLogConsoleLevel EnLogLevel_Info
#define EnLogFileLevel EnLogLevel_Debug

#define DEBUGPRINT(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Debug ? 1 : 0, EnLogFileLevel <= EnLogLevel_Debug ? 1 : 0, EnLogFile_Debug, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define LOGPRINT(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Info ? 1 : 0, EnLogFileLevel <= EnLogLevel_Info ? 1 : 0, EnLogFile_Info, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define LOGPRINT_NOPREFIX(x, ...) EnLogNoPrefix(EnLogConsoleLevel <= EnLogLevel_Info ? 1 : 0, EnLogFileLevel <= EnLogLevel_Info ? 1 : 0, EnLogFile_Info, x, ##__VA_ARGS__)
#define ERRORPRINT(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Error ? 1 : 0, EnLogFileLevel <= EnLogLevel_Error ? 1 : 0, EnLogFile_Error, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)

#define DEBUGPRINT_THROTTLE(pTNextMessage, throttle, x, ...) EnLogThrottle(pTNextMessage, throttle, EnLogConsoleLevel <= EnLogLevel_Debug ? 1 : 0, EnLogFileLevel <= EnLogLevel_Debug ? 1 : 0, EnLogFile_Debug, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define LOGPRINT_THROTTLE(pTNextMessage, throttle, x, ...) EnLogThrottle(pTNextMessage, throttle, EnLogConsoleLevel <= EnLogLevel_Info ? 1 : 0, EnLogFileLevel <= EnLogLevel_Info ? 1 : 0, EnLogFile_Info, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define ERRORPRINT_THROTTLE(pTNextMessage, throttle, x, ...) EnLogThrottle(pTNextMessage, throttle, EnLogConsoleLevel <= EnLogLevel_Error ? 1 : 0, EnLogFileLevel <= EnLogLevel_Error ? 1 : 0, EnLogFile_Error, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)

#define DEBUGPRINT_NOCONSOLE(x, ...) EnLog(0, EnLogFileLevel <= EnLogLevel_Debug ? 1 : 0, EnLogFile_Debug, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define LOGPRINT_NOCONSOLE(x, ...) EnLog(0, EnLogFileLevel <= EnLogLevel_Info ? 1 : 0, EnLogFile_Info, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define ERRORPRINT_NOCONSOLE(x, ...) EnLog(0, EnLogFileLevel <= EnLogLevel_Error ? 1 : 0, EnLogFile_Error, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)

#define DEBUGPRINT_NOCONSOLE_THROTTLE(pTNextMessage, throttle, x, ...) EnLogThrottle(pTNextMessage, throttle, 0, EnLogFileLevel <= EnLogLevel_Debug ? 1 : 0, EnLogFile_Debug, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define LOGPRINT_NOCONSOLE_THROTTLE(pTNextMessage, throttle, x, ...) EnLog(ThrottlepTNextMessage, throttle, 0, EnLogFileLevel <= EnLogLevel_Info ? 1 : 0, EnLogFile_Info, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define ERRORPRINT_NOCONSOLE_THROTTLE(pTNextMessage, throttle, x, ...) EnLogThrottle(pTNextMessage, throttle, 0, EnLogFileLevel <= EnLogLevel_Error ? 1 : 0, EnLogFile_Error, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)

#define DEBUGPRINT_NOFILE(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Debug ? 1 : 0, 0, EnLogFile_Debug, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define LOGPRINT_NOFILE(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Info ? 1 : 0, 0, EnLogFile_Info, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define ERRORPRINT_NOFILE(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Error ? 1 : 0, 0, EnLogFile_Error, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)

#define DEBUGPRINT_NOFILE_THROTTLE(pTNextMessage, throttle, x, ...) EnLogThrottle(pTNextMessage, throttle, EnLogConsoleLevel <= EnLogLevel_Debug ? 1 : 0, 0, EnLogFile_Debug, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define LOGPRINT_NOFILE_THROTTLE(pTNextMessage, throttle, x, ...) EnLogThrottle(pTNextMessage, throttle, EnLogConsoleLevel <= EnLogLevel_Info ? 1 : 0, 0, EnLogFile_Info, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define ERRORPRINT_NOFILE_THROTTLE(pTNextMessage, throttle, x, ...) EnLogThrottle(pTNextMessage, throttle, EnLogConsoleLevel <= EnLogLevel_Error ? 1 : 0, 0, EnLogFile_Error, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)

// Player logging
#define PLAYER_DEBUGPRINT(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Debug ? 1 : 0, EnLogFileLevel <= EnLogLevel_Debug ? 1 : 0, EnLogFile_Player, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define PLAYER_LOGPRINT(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Info ? 1 : 0, EnLogFileLevel <= EnLogLevel_Info ? 1 : 0, EnLogFile_Player, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define PLAYER_ERRORPRINT(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Error ? 1 : 0, EnLogFileLevel <= EnLogLevel_Error ? 1 : 0, EnLogFile_Player, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)

#define PLAYER_DEBUGPRINT_NOCONSOLE(x, ...) EnLog(0, EnLogFileLevel <= EnLogLevel_Debug ? 1 : 0, EnLogFile_Player, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define PLAYER_LOGPRINT_NOCONSOLE(x, ...) EnLog(0, EnLogFileLevel <= EnLogLevel_Info ? 1 : 0, EnLogFile_Player, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define PLAYER_ERRORPRINT_NOCONSOLE(x, ...) EnLog(0, EnLogFileLevel <= EnLogLevel_Error ? 1 : 0, EnLogFile_Player, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)

#define PLAYER_DEBUGPRINT_NOFILE(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Debug ? 1 : 0, 0, EnLogFile_Player, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define PLAYER_LOGPRINT_NOFILE(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Info ? 1 : 0, 0, EnLogFile_Player, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define PLAYER_ERRORPRINT_NOFILE(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Error ? 1 : 0, 0, EnLogFile_Player, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)

// TSSource logging
#define TSSOURCE_DEBUGPRINT(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Debug ? 1 : 0, EnLogFileLevel <= EnLogLevel_Debug ? 1 : 0, EnLogFile_TSSource, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define TSSOURCE_LOGPRINT(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Info ? 1 : 0, EnLogFileLevel <= EnLogLevel_Info ? 1 : 0, EnLogFile_TSSource, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define TSSOURCE_ERRORPRINT(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Error ? 1 : 0, EnLogFileLevel <= EnLogLevel_Error ? 1 : 0, EnLogFile_TSSource, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)

#define TSSOURCE_DEBUGPRINT_NOCONSOLE(x, ...) EnLog(0, EnLogFileLevel <= EnLogLevel_Debug ? 1 : 0, EnLogFile_TSSource, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define TSSOURCE_LOGPRINT_NOCONSOLE(x, ...) EnLog(0, EnLogFileLevel <= EnLogLevel_Info ? 1 : 0, EnLogFile_TSSource, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define TSSOURCE_ERRORPRINT_NOCONSOLE(x, ...) EnLog(0, EnLogFileLevel <= EnLogLevel_Error ? 1 : 0, EnLogFile_TSSource, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)

#define TSSOURCE_DEBUGPRINT_NOFILE(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Debug ? 1 : 0, 0, EnLogFile_TSSource, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define TSSOURCE_LOGPRINT_NOFILE(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Info ? 1 : 0, 0, EnLogFile_TSSource, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)
#define TSSOURCE_ERRORPRINT_NOFILE(x, ...) EnLog(EnLogConsoleLevel <= EnLogLevel_Error ? 1 : 0, 0, EnLogFile_TSSource, ensys, ensubsys, __LINE__, x, ##__VA_ARGS__)

#endif
