/*
 * Copyright © 2018-2019 Koherence, LLC (www.koherence.com).
 * All Rights Reserved.
 */

#ifndef LIBENUTILS_TASKLET_H
#define LIBENUTILS_TASKLET_H

typedef uint64_t TaskletHandle_t;
#define INVALID_TASKLET_HANDLE 0
typedef struct _TaskletRunner *TaskletRunnerHandle_t;
typedef union
{
    void *pArg;
    int64_t arg64;
    uint8_t vRaw[0];
} TaskletArg;

typedef void (*TaskletFunction)(TaskletRunnerHandle_t hTaskletRunner,
    TaskletArg *pArg);

typedef struct _Tasklet
{
    TaskletFunction Function;
    TaskletArg *pArg;
} Tasklet;

#define TASKLET_PRIORITY_LOW  0
#define TASKLET_PRIORITY_NORMAL 1
#define TASKLET_PRIORITY_HIGH  2

#define TASKLET_PRIORITY_REALTIME_LOW 100
#define TASKLET_PRIORITY_REALTIME_MEDIUM 101
#define TASKLET_PRIORITY_REALTIME_HIGH 102

int32_t
ENUtils_TaskletRunnerCreate(char *pName, int32_t priority,
    int32_t taskletUserDataBytes, int32_t nTasklets,
    TaskletRunnerHandle_t *phTaskletRunner);

int32_t
ENUtils_TaskletRunnerDestroy(TaskletRunnerHandle_t hTaskletRunner);

int32_t
ENUtils_TaskletRunnerRunAllTasklets(TaskletRunnerHandle_t hTaskletRunner);

#define ENUtils_TaskletRunnerAddTasklet(hTaskletRunner, Tasklet, PHandle) ENUtils_TaskletRunnerAddTaskletMarked(hTaskletRunner, Tasklet, __FILE__, __FUNCTION__, __LINE__, PHandle)

#define ENUtils_TaskletRunnerAddTaskletWithExecutionTime(hTaskletRunner, Tasklet, ExecutionTime, PHandle) ENUtils_TaskletRunnerAddTaskletWithExecutionTimeMarked(hTaskletRunner, Tasklet, ExecutionTime, __FILE__, __FUNCTION__, __LINE__, PHandle)

int32_t
ENUtils_TaskletRunnerTaskletAllocate(TaskletRunnerHandle_t hTaskletRunner,
    int32_t taskletUserDataBytes, Tasklet **ppTasklet);

void
ENUtils_TaskletRunnerTaskletReturn(TaskletRunnerHandle_t hTaskletRunner,
    Tasklet *pTasklet);

int32_t
ENUtils_TaskletRunnerAddTaskletMarked(TaskletRunnerHandle_t hTaskletRunner,
    Tasklet *pTasklet, const char *pFile, const char *pFunction, int32_t line,
    TaskletHandle_t *phTasklet);

int32_t
ENUtils_TaskletRunnerAddTaskletWithExecutionTimeMarked(
    TaskletRunnerHandle_t hTaskletRunner, Tasklet *pTasklet,
    struct timespec runAtMonoTime, const char *pFile, const char *pFunction,
    int32_t line, TaskletHandle_t *phTasklet);

int32_t
ENUtils_TaskletRunnerCancelTasklet(TaskletRunnerHandle_t hTaskletRunner,
    TaskletHandle_t hTasklet);

// Search for a tasklet with the specified Function and return the tasklet if
// found. Note this can only be safely called from within a tasklet function.
int32_t
ENUtils_TaskletRunnerPeek(TaskletRunnerHandle_t hTaskletRunner,
    TaskletFunction Function, TaskletArg **ppArg, TaskletHandle_t *phTasklet);

// Search for a tasklet with the specified function and return 1 if a tasklet
// is found, 0 otherwise. The tasklet is guaranted to not be executing when
// the check is performed, but could be executing on return.
int32_t
ENUtils_TaskletRunnerCheckPending(TaskletRunnerHandle_t hTaskletRunner,
    TaskletFunction Function);

#endif
